/*
 * canvas_actions.c : Actions for handling events generated in the canvas widget 
 *
 * This file defines the global actionTable used in XtAppAddActions() and
 * defines the action procedures, except those used on the settingsPanel
 * which are imported. Eventually there may be more actions, and they'll
 * be put here.
 *
 * Thanks to:
 * George Ferguson, ferguson@cs.rochester.edu, 12 Sep 1991.
 *
 */

#include "geotraceman.h"

/*XtAppContext appContext;*/

/*
 * Functions defined here
 */
void addCanvasActions();
void draw_bounding_box();
static void CanvasConfigureAction(), CanvasExposeAction();
static void CanvasButtonDownAction(), CanvasButtonUpAction();
static void CanvasButtonDownMotionAction();

/*
 * Data defined here:
 */
static XtActionsRec Canvas_Action_Table[] = {
	{"Canvas_Expose", CanvasExposeAction},
	{"Canvas_Configure", CanvasConfigureAction},
	{"Canvas_ButtonDown", CanvasButtonDownAction},
	{"Canvas_ButtonUp", CanvasButtonUpAction},
	{"Canvas_ButtonDownMotion", CanvasButtonDownMotionAction},
};

static Bounding_Box_Struct boxdata = {0,0,0,0,0,0,0,0};

/*	-	-	-	-	-	-	-	-	*/


/*
Registers the canvas actions with the server
*/
void 
add_canvas_actions( app_context)
	XtAppContext app_context;
{
	XtAppAddActions(app_context,Canvas_Action_Table,XtNumber(Canvas_Action_Table));
}

/*
Passed the mouse event and the structure giving the pixel coordinate limits of the last bounding
box, calculates the new bounding box limits, and outputs it to the canvas.
*/
void
draw_bounding_box(boxdata, buttonEvent)
	Bounding_Box_Struct *boxdata;
	XButtonEvent *buttonEvent;
{
	Drawable drawable;
	int size;

	/* Grab the pixel values from the button event */
	boxdata->currentx = buttonEvent->x;
	boxdata->currenty = buttonEvent->y;

	/* Get the handle of the drawable */
	drawable = get_drawable();	

	/* Calculate the size of the zoom box */
	if (abs(boxdata->currentx-boxdata->origx) > abs(boxdata->currenty-boxdata->origy))
		size = abs(boxdata->currentx - boxdata->origx);
	else 
		size = abs(boxdata->currenty - boxdata->origy);

	/* upper left quadrant */
	if ((boxdata->currentx < boxdata->origx) && (boxdata->currenty < boxdata->origy)) {
		boxdata->finalx = boxdata->origx;
		boxdata->finaly = boxdata->origy;
		boxdata->startx = boxdata->origx-size;
		boxdata->starty = boxdata->origy-size;
	}
	/* lower left quadrant */
	else if ((boxdata->currentx < boxdata->origx) && (boxdata->currenty > boxdata->origy)) {
		boxdata->finalx = boxdata->origx;
		boxdata->finaly = boxdata->origy+size;
		boxdata->startx = boxdata->origx-size;
		boxdata->starty = boxdata->origy;
	}
	/* upper right quadrant */
	else if ((boxdata->currentx > boxdata->origx) && (boxdata->currenty < boxdata->origy)) {
		boxdata->finalx = boxdata->origx+size;
		boxdata->finaly = boxdata->origy;
		boxdata->startx = boxdata->origx;
		boxdata->starty = boxdata->origy-size;
	}
	/* lower right quadrant */
	else {
		boxdata->finalx = boxdata->origx + size;
		boxdata->finaly = boxdata->origy + size;
		boxdata->startx = boxdata->origx;
		boxdata->starty = boxdata->origy;
	}

	/* Draw the rectangle to the canvas */
	XDrawRectangle(display, drawable, gc_inverse, boxdata->startx, boxdata->starty, size, size);
}
 
ACTION (CanvasExposeAction)
{
        swapbuffers();
}

ACTION (CanvasConfigureAction)
{
	/* Widget canvas; */
	XConfigureEvent *xconfigure = (XConfigureEvent *) event;
	int width = xconfigure->width, height = xconfigure->height;

	vo_xt_win_size (width + 1, height + 1);

	screen_width = width;
	screen_height = height;
/*printf("Reconfigured height is :  %d\n", height);*/
/*printf("Reconfigured width is :  %d\n", width);*/
   
	reshapeviewport ();
	update_display(); 
}

ACTION (CanvasButtonDownAction)
{
	Widget tmpWidget;
	XButtonEvent *buttonEvent = (XButtonEvent *)event;
	int x=buttonEvent->x, y=buttonEvent->y;
	
        if ((tmpWidget = XtNameToWidget( toplevel, "*mstatusnLabel")) == NULL)
                fail0("No such widget *mstatusnLabel");
        setLabel(tmpWidget , "zooming");

	boxdata.origx = x;
	boxdata.origy = y;
}

ACTION (CanvasButtonUpAction)
{
	XButtonEvent *buttonEvent = (XButtonEvent *)event;
	Widget tmpWidget;

	/* save the current zoom details for the undo option */
	UndoMap(ZOOM_SAVE);
	draw_bounding_box(&boxdata, buttonEvent);

	if (process_zoom(boxdata) == 0) {
		update_display();
	}
	else {
		swapbuffers();
		DisplayMessage(toplevel, "Error", "Zoom level is not possible");
        	if ((tmpWidget = XtNameToWidget( toplevel, "*mstatusnLabel")) == NULL)
	                fail0("No such widget *mstatusnLabel");
	        setLabel(tmpWidget , "waiting");
	}
}

ACTION (CanvasButtonDownMotionAction)
{
	XButtonEvent *buttonEvent = (XButtonEvent *)event;

	if ((boxdata.origx == 0) && (boxdata.origy == 0)) {
		boxdata.origx = buttonEvent->x;
		boxdata.origy = buttonEvent->y;
	}
	swapbuffers();
	draw_bounding_box(&boxdata, buttonEvent);
}
